/*
 * Copyright (c) 1996-1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifdef VTK

#ifndef MAKEDEPEND
#include <vtkActorCollection.h>
#include <vtkCamera.h>
#include <vtkRendererCollection.h>
#include <vtkRenderWindow.h>
#endif

#include <DemVtk/algebra3.h>

#include <DemUnidraw/demcmds.h>
#include <DemUnidraw/demeditor.h>
#include <DemUnidraw/demkit.h>
#include <DemUnidraw/demviewer.h>
#include <DemUnidraw/lodactor.h>

#include <TopoFace/fgeomobjs.h>

#include <OverlayUnidraw/ovcomps.h>
#include <OverlayUnidraw/ovdamage.h>
#include <OverlayUnidraw/oved.h>
#include <OverlayUnidraw/ovselection.h>
#include <OverlayUnidraw/ovviewer.h>
#include <OverlayUnidraw/ovviews.h>

#include <UniIdraw/idclasses.h>
#include <UniIdraw/ided.h>

#include <Unidraw/catalog.h>
#include <Unidraw/iterator.h>
#include <Unidraw/manips.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/Commands/command.h>
#include <Unidraw/Graphic/graphic.h>
#include <Unidraw/Tools/tool.h>
#include <InterViews/canvas.h>
#include <InterViews/cursor.h>
#include <InterViews/display.h>
#include <InterViews/perspective.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>
#include <IV-2_6/InterViews/painter.h>
#include <IV-2_6/InterViews/sensor.h>
#include <Attribute/attribute.h>
#include <Attribute/attrlist.h>
#include <Attribute/attrvalue.h>
#ifdef HAVE_ACE
#include <ace/SOCK_Stream.h>
#include <ace/SOCK_Connector.h>
#include <ace/INET_Addr.h>
#endif

#include <math.h>

DemViewer::DemViewer (
    Editor* ed, GraphicView* gv, UPage* page, Grid* grid, 
    Coord w, Coord h, Orientation orientation,
    Alignment align, Zooming zoom
) : OverlayViewer(ed, gv, page, grid, w, h, orientation) {
    _renWindow = nil;
    _ren = nil;
    _capture_on = false;
    _rte_capture = false;
    _framenum = 0;
    _lastxyset = false;
    _lastx = _lasty = 0.0;
}

DemViewer::~DemViewer () {}

void DemViewer::UpdateMagnifVar() {}

void DemViewer::Resize()
{
  //    OverlayViewer::Resize();
    Viewer::Resize();
    PrepareDoubleBuf();

    if (!_ren) {
      _ren = vtkRenderer::New();
	_renWindow = vtkRenderWindow::New();
	_renWindow->AddRenderer(_ren);
	_ren->SetBackground(20.0/255.0, 191.0/255.0, 255.0/255.0);
	DemInitCmd* initcmd = new DemInitCmd(nil);
	initcmd->SetEditor(_editor);
	initcmd->Execute();
    }
    int pwidth = canvas->pwidth();
    int pheight = canvas->pheight();
    _renWindow->SetSize(pwidth, pheight);

    DemKit* kit = DemKit::Instance();
    kit->SetXIds(_renWindow, canvas);

    /* initial drawing commands, if any */
    if (_needs_resize) {
      ((OverlayEditor*)_editor)->InitCommands();
      _needs_resize = false;
    }
}


void DemViewer::StartBuffering() 
{
}

void DemViewer::FinishBuffering(boolean refresh_needed) 
{
}

void DemViewer::AddActor(LODActor* a) {
  if (_ren) {
    Catalog* catalog = unidraw->GetCatalog();
    Attribute* lod = ((OverlayComp*)GetEditor()->GetComponent())->GetAttributeList()->GetAttr("lod");
    if (lod) {
      //const char* lod = catalog->GetAttribute("lod");
      //if (strcmp(lod, "true")==0) {
      int ax, ay;
      a->GetXY(ax, ay);
      vtkActorCollection* actors = _ren->GetActors();
      vtkActor* anActor;
      for (actors->InitTraversal(); (anActor = actors->GetNextItem()); ) {
	int ix, iy;
	((LODActor*)anActor)->GetXY(ix, ix);
	if (ix == ax && iy == ay)
	  return;
      }
    }
    _ren->AddActor(a);
  }
}

void DemViewer::Draw() 
{
    OverlaySelection* s = (OverlaySelection*)GetSelection();

    StartBuffering();
    s->ShowHighlights(this);

//  guts of GraphicBlock::Draw();
    if (canvas != nil) {
	vtkActorCollection* actcol = _ren->GetActors();
	UpdateCameraFootprint();
	if (actcol->GetNumberOfItems()==0)
	    output->ClearRect(canvas, 0, 0, xmax, ymax);
	else {
	    _renWindow->Render();
	}
	if (_capture_on) {
	    char fn[40];
	    sprintf(fn, "frame%.4d.ppm", _framenum++);
	    _renWindow->SetFileName(fn);
	    _renWindow->SaveImageAsPPM();
	}
#if 1
	if (_graphic != nil) 
	    _graphic->Draw(canvas, 0, 0, xmax, ymax);
#endif
    }

    FinishBuffering(true);

    s->Init(this);
    s->ShowHandles(this);

    _damage->Reset();
    if (GetEditor()->GetWindow()->cursor() != arrow)
        GetEditor()->GetWindow()->cursor(arrow);
}

void DemViewer::Redraw(Coord x0, Coord y0, Coord x1, Coord y1) 
{
    OverlaySelection* s = (OverlaySelection*)GetSelection();

    StartBuffering();
    s->ShowHighlights(this);

// guts of GraphicBlock::Redraw(x0, y0, x1, y1);
    if (canvas != nil) {
	vtkRenderer* renderer = _ren;
	vtkActorCollection* actcol = renderer->GetActors();
	if (actcol->GetNumberOfItems()==0)
	    output->ClearRect(canvas, x0, y0, x1, y1);
	else {
	    UpdateCameraFootprint();
	    _renWindow->Render();
	}
	if (_capture_on) {
	    char fn[40];
	    sprintf(fn, "frame%.4d.ppm", _framenum++);
	    _renWindow->SetFileName(fn);
	    _renWindow->SaveImageAsPPM();
	}
#if 1
	if (_graphic != nil) 
	    _graphic->DrawClipped(canvas, x0, y0, x1, y1);
#endif
    }

    FinishBuffering(true);

    xorPainter->Clip(canvas, x0, y0, x1, y1);
    s->ShowHandles(this);
    xorPainter->NoClip();
    if (GetEditor()->GetWindow()->cursor() != arrow)
        GetEditor()->GetWindow()->cursor(arrow);
}

void DemViewer::UpdateCameraFootprint() {
  float cx[3], cy[3];
  float cameraPos[3], cameraFP[3];
  vtkRenderer* ren = _ren;
  vtkCamera* camera = ren->GetActiveCamera();
  camera->GetPosition((float *)cameraPos);
  camera->GetFocalPoint((float *)cameraFP);

  cx[0] = cameraPos[0];
  cy[0] = cameraPos[1];
  vec2 camvec(cameraFP[0] - cameraPos[0], cameraFP[1] - cameraPos[1]);
  camvec.normalize();
  float range[2];
  camera->GetClippingRange(range);
  // camvec *= range[1];

  float anglev = camera->GetViewAngle();
  float angle3 = 90.0 - anglev/2.0;
  float lena = (range[1] / sin(angle3 * (M_PI / 180.0))); // times sin 90 = 1
  //  float lenc = (range[1] / sin(angle3 * (M_PI / 180.0))) 
  //    * sin((anglev/2.0)/(M_PI / 180.0));

  mat3 leftrot = rotation2D(vec2(0.,0.), anglev/2.0);
  vec2 leftvec = leftrot.inverse() * camvec;
  leftvec *= lena;
  mat3 riterot = rotation2D(vec2(0.,0.), -anglev/2.0);
  vec2 ritevec = riterot.inverse() * camvec;
  ritevec *= lena;

  cx[1] = cx[0] + leftvec[0];
  cy[1] = cy[0] + leftvec[1];
  cx[2] = cx[0] + ritevec[0];
  cy[2] = cy[0] + ritevec[1];

  LODActor::SetFootprint(cx, cy);
  // remove all actors
  //vtkActorCollection* actcol = _ren->GetActors();
  //actcol->RemoveAllItems();
  Catalog* catalog = unidraw->GetCatalog();
  Attribute* lod = ((OverlayComp*)GetEditor()->GetComponent())->GetAttributeList()->GetAttr("lod");
  if (lod) {
    //const char* lod = catalog->GetAttribute("lod");
    //if (strcmp(lod, "true")==0) {
    // load visible actors
    ((DemEditor*)_editor)->LoadActorsByFootprint(cx, cy);
  }
#if 0
#ifdef HAVE_ACE
  FFillPolygonObj camview(cx, cy, 3);
  FBoxObj cvbox;
  camview.GetBox(cvbox);
  char buf[256];
  sprintf(buf,
"mapfcl(\n gs(:fgcolor \"blue\",0,0,0 :brush 65535,0 :fillbg 0 :nonepat),\n\
area((%0.f,%0.f),(%0.f,%0.f),(%0.f,%0.f) :gs 0) :sw_e %0.f :sw_n %0.f)",
	  cx[0], cy[0], cx[1], cy[1], cx[2], cy[2], cvbox._left, cvbox._bottom);
  printf(buf);
  ACE_SOCK_Stream* _socket = new ACE_SOCK_Stream;
  ACE_SOCK_Connector* connector = new ACE_SOCK_Connector;
  ACE_INET_Addr* addr = new ACE_INET_Addr(20011, "localhost");

  if (connector->connect (*_socket, *addr) == -1) {
    ACE_ERROR ((LM_ERROR, "%p\n", "open"));
    return;
  }
  if (_socket->send (buf, strlen(buf)) == -1) {
    ACE_ERROR ((LM_ERROR, "%p\n", "send"));
    return;
  }
  _socket->close();
#endif
#endif
}

#endif
