/*
 * Copyright 1997,1998 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <DlgUnidraw/dlgcomps.h>
#include <DlgUnidraw/dlgcreator.h>
#include <DlgUnidraw/dlgfunc.h>
#include <MapUnidraw/mapcmds.h>
#include <Dlg/dlgdb.h>
#include <Dlg/dlglib.h>
#include <Dlg/dlgcov.h>
#include <Dlg/dlgfclass.h>
#include <Dlg/dlgfeature.h>

/*===========================================================================*/

#ifdef DLG_FREE /* if not already defined in MapServ */

Load100KDLGFCFunc::Load100KDLGFCFunc(ComTerp* comterp) : ComFunc(comterp) {
}

void Load100KDLGFCFunc::execute() {
    ComValue dbpath = stack_arg(0);
    ComValue libname = stack_arg(1);
    ComValue covname = stack_arg(2);
    ComValue fcname = stack_arg(3);
    reset_stack();
    ComValue result;

    Creator::instance(new DlgCreator());
    const char* dbdir = dbpath.string_ptr();
    DlgDatabase* db = new DlgDatabase(dbdir);
    DlgLibrary* lib = db->library(libname.string_ptr());
    DlgDbComp* dbcomp = new DlgDbComp(db);
    MapShowCmd* showcmd = new MapShowCmd();
    showcmd->SetUseEditor(false);
    showcmd->SetLibName(libname.string_ptr());
    showcmd->SetCovName(covname.string_ptr());
    showcmd->SetFClassName(fcname.string_ptr());
    dbcomp->Interpret(showcmd);

    DlgLibComp* libcomp = (DlgLibComp*)dbcomp->FindLibComp(lib);
    DlgCoverage* cov = lib->coverage(covname.string_ptr());
    DlgCovComp* covcomp = (DlgCovComp*)libcomp->FindCovComp(cov);
    DlgFeatureClass* fclass = cov->feature_class(fcname.string_ptr());
    DlgFClassComp* fccomp = (DlgFClassComp*)covcomp->FindFClassComp(fclass);
    result.type(AttributeValue::ObjectType);
    result.obj_ref() = fccomp;
    push_stack(result);
}

#endif
