#ifndef NOXVT_H
#define NOXVT_H 1

#include <ctype.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdarg.h>

#define SZ_FNAME 256

typedef struct s_dir {         /* DIRECTORY - directory descriptor */
    char path[SZ_FNAME + 5];
} DIRECTORY, *DIRECTORY_PTR;

typedef struct s_filespec {     /* file specification */
    DIRECTORY dir;              /* directory */
    char type[6];               /* file type or extension */
    char name[SZ_FNAME + 1];    /* file name (may be partial path) */
    char creator[6];            /* file creator */
} FILE_SPEC, *FILE_SPEC_PTR;

typedef enum e_file {           /* result from file open & save dialogs */
    FL_BAD,                     /* error occurred */
    FL_CANCEL,                  /* cancel button clicked */
    FL_OK                       /* OK button clicked */
} FL_STATUS;

typedef enum e_ask_resp {       /* response from ask fcn */
    RESP_DEFAULT,               /* default button */
    RESP_2,                     /* second button */
    RESP_3                      /* third button */
} ASK_RESPONSE;

#define TRUE 1
#define FALSE 0

#define NOREF(a)	a = a

#define EOL_SEQ		"\n"

/* NRL XVT overrides */

#ifndef BOOLEAN
#define BOOLEAN short
#endif

typedef char          *DATA_PTR; /* ptr to arbitrary data */
typedef long           WINDOW;   /* window descriptor */
typedef unsigned long  COLOR;  	 /* color encapsulation */

#ifndef COLOR_RED
#define COLOR_RED       0x01FF0000L
#endif
#ifndef COLOR_GREEN
#define COLOR_GREEN     0x0200FF00L
#endif
#ifndef COLOR_BLUE
#define COLOR_BLUE      0x030000FFL
#endif
#ifndef COLOR_CYAN
#define COLOR_CYAN      0x0400FFFFL
#endif
#ifndef COLOR_MAGENTA
#define COLOR_MAGENTA   0x05FF00FFL
#endif
#ifndef COLOR_YELLOW
#define COLOR_YELLOW    0x06FFFF00L
#endif
#ifndef COLOR_BLACK
#define COLOR_BLACK     0x07000000L
#endif
#ifndef COLOR_DKGRAY
#define COLOR_DKGRAY    0x08404040L
#endif
#ifndef COLOR_GRAY
#define COLOR_GRAY      0x09808080L
#endif
#ifndef COLOR_LTGRAY
#define COLOR_LTGRAY    0x0AC0C0C0L
#endif
#ifndef COLOR_WHITE
#define COLOR_WHITE     0x0BFFFFFFL
#endif
#define XVT_MAKE_COLOR(r,g,b) ((COLOR)((((unsigned long)(r)&0xFF) << 16) | \
				       (((unsigned long)(g)&0xFF) << 8)  | \
				       (((unsigned long)(b)&0xFF))))

typedef struct s_pnt {          /* mathematical point */
    short v;                    /* vertical (y) coordinate */
    short h;                    /* horizontal (x) coordinate */
} PNT, *PNT_PTR;

/***************************************************************
@    MRCT
****************************************************************
MUSE version of XVT RCT.
*/
typedef struct RCT
{
    short top;      /* top coord.   */
	short left;     /* left coord. */
    short bottom;   /* bottom coord.   */
	short right;    /* right coord. */
}               RCT;	 
typedef void* GHANDLE;

#define XVT_CC_PROTO 1

#define ULONG_MAX (unsigned long)(~0L)

#ifndef min
#define min(x, y) ((x) < (y) ? (x) : (y))
#endif

typedef struct s_win_def {
    long dummy;
} WIN_DEF;

#define xvt_gmem_malloc(N) malloc(N)
#define xvt_dm_post_fatal_exit(S) { fprintf(stderr, S); fprintf(stderr, "\n"); exit(-1); }

#endif
