/*
 * Copyright (c) 1996 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <MapUnidraw/mapcamera.h>
#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapviewstate.h>

#include <OverlayUnidraw/ovarrow.h>
#include <OverlayUnidraw/ovselection.h>
#include <OverlayUnidraw/ovshowhide.h>

#include <UniIdraw/idarrows.h>

#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/manips.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/viewer.h>
#include <Unidraw/Commands/macro.h>
#include <Unidraw/Graphic/damage.h>

#include <InterViews/cursor.h>
#include <InterViews/rubline.h>
#include <InterViews/rubverts.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

#include <stdio.h>

static const unit = 15;
static Coord xOpen[] = { 0, unit/2, unit/2, unit };
static Coord yOpen[] = { 0, unit/4, unit*3/4, unit };
static const int nOpen = 4;

ClassId CameraPosTool::GetClassId () { return CAMERAPOS_TOOL; }

boolean CameraPosTool::IsA (ClassId id) {
    return CAMERAPOS_TOOL == id || GraphicCompTool::IsA(id);
}

Tool* CameraPosTool::Copy () { return new CameraPosTool(CopyControlInfo()); }

CameraPosTool::CameraPosTool(ControlInfo* info)
:GraphicCompTool()
{
    if (info)
	SetControlInfo(info);
    ArrowLine* ml = new ArrowLine(
        0, 0, unit, unit, true, false, 2., stdgraphic
    );
    Catalog* catalog = unidraw->GetCatalog();
    PSBrush* br = catalog->FindBrush(0xffff, 2);
    ml->SetBrush(br);
    PSColor* col = catalog->FindColor("Red");
    ml->SetColors(col, col);
    ArrowLineOvComp* mlComp = new ArrowLineOvComp(ml);
    Init(mlComp);
}

Command* CameraPosTool::InterpretManipulator (Manipulator* m) {
    DragManip* dm = (DragManip*) m;
    MapEditor* ed = (MapEditor*) dm->GetViewer()->GetEditor();
    Transformer* rel = dm->GetTransformer();
    MacroCmd* cmd = new MacroCmd(ed);

    RubberLine* rl = (RubberLine*) dm->GetRubberband();

    Coord px0, py0, px1, py1;
    int pt;
    rl->GetCurrent(px0, py0, px1, py1);

    if (rel != nil) {
      rel = new Transformer(rel);
      rel->Invert();
    }

    if (px0 != px1 || py0 != py1) {
        float fpx, fpy, ffx, ffy;
	rel->transform(px0, py0, fpx, fpy);
	rel->transform(px1, py1, ffx, ffy);

	// get line in ground coordinates
	// inverse project to obtain radian-based lat/long
	// set camera to those coordinates
	MapViewState* mapviewstate = ed->GetMapViewState();
	float dpx, dpy, dfx, dfy;
	if (mapviewstate && mapviewstate->deftrans()) {
	  mapviewstate->deftrans()->inverse_transform(fpx, fpy, dpx, dpy);
	  mapviewstate->deftrans()->inverse_transform(ffx, ffy, dfx, dfy);
	} else {
	  dpx = fpx; dpy = fpy; dfx = ffx; dfy = ffy;
	}

	PJ* proj = ed->GetProjection();
	if (!proj) // utmviewer
	    proj = mapviewstate->implicitproj();
	if (proj) {
	  float gpx, gpy, gfx, gfy;
	  UV data;
	  data.u = dpx;
	  data.v = dpy;
	  data = pj_inv(data, proj);
	  gpx = data.u;
	  gpy = data.v;

	  data.u = dfx;
	  data.v = dfy;
	  data = pj_inv(data, proj);
	  gfx = data.u;
	  gfy = data.v;

	  //	  printf("pos: x=%f, y=%f, degx=%f, degy=%f\n", gpx, gpy,
	  //		 gpx * RAD_TO_DEG, gpy * RAD_TO_DEG);
	  //	  printf("fpt: x=%f, y=%f, degx=%f, degy=%f\n", gfx, gfy,
	  //		 gfx * RAD_TO_DEG, gfy * RAD_TO_DEG);

	  if (ed->GetCameraComp()) {
	      Viewer* viewer = ed->GetViewer();    
	      OverlayView* view =
		((OverlayComp*)ed->GetCameraComp())->FindView(ed->GetViewer());
	      ArrowLineOvComp* comp = (ArrowLineOvComp*)ed->GetCameraComp();
	      ArrowLine* aline = comp->GetArrowLine();
	      aline->SetTransformer(rel);
//	      aline->SetOriginal(px0, py0, px1, py1);

	      //ArrowLine* alinev = (ArrowLine*)(view->GetGraphic());
	      ArrowLine* alinev = (ArrowLine*)(ed->GetCameraComp()->GetGraphic());
	      viewer->GetDamage()->Incur(alinev);
	      alinev->SetTransformer(rel);
	      alinev->SetOriginal(px0, py0, px1, py1);
	      viewer->GetDamage()->Incur(alinev);
	      unidraw->Update();
	  }
	  else {
	      Catalog* catalog = unidraw->GetCatalog();
	      ArrowLine* aline = new ArrowLine(px0, py0, px1, py1, false, true,
					       1.25, stdgraphic);
	      PSBrush* br = catalog->FindBrush(0xffff, 2);
	      aline->SetBrush(br);
	      PSColor* col = catalog->FindColor("Red");
	      aline->SetColors(col, col);
	      aline->SetTransformer(rel);
	      ArrowLineOvComp* comp = new ArrowLineOvComp(aline);
	      Command* pcmd = new PasteCmd(ed, new Clipboard(comp));
	      pcmd->Execute();
	      ed->GetSelection()->Clear();
	      ed->SetCameraComp(comp);
	  }
	  Command* ccmd = new SendCameraCmd(gpx, gpy, gfx, gfy);
	  ccmd->SetEditor(ed);
	  cmd->Append(ccmd);
	}
	else
	  printf("projection is nil\n");
    }
    return cmd;
}

/****************************************************************************/

ClassId CameraPathTool::GetClassId () { return CAMERAPATH_TOOL; }

boolean CameraPathTool::IsA (ClassId id) {
    return CAMERAPATH_TOOL == id || GraphicCompTool::IsA(id);
}

Tool* CameraPathTool::Copy () { return new CameraPathTool(CopyControlInfo()); }

CameraPathTool::CameraPathTool(ControlInfo* info)
:GraphicCompTool()
{
    if (info)
	SetControlInfo(info);
    ArrowMultiLine* ml = new ArrowMultiLine(
        xOpen, yOpen, nOpen, true, false, 1., stdgraphic
    );
    ArrowMultiLineOvComp* mlComp = new ArrowMultiLineOvComp(ml);
    Init(mlComp);
}

Command* CameraPathTool::InterpretManipulator (Manipulator* m) {
    VertexManip* dm = (VertexManip*) m;
    MapEditor* ed = (MapEditor*) dm->GetViewer()->GetEditor();
    Transformer* rel = dm->GetTransformer();
    MacroCmd* cmd = new MacroCmd(ed);

    GrowingVertices* gv = (GrowingVertices*) dm->GetRubberband();
    Catalog* catalog = unidraw->GetCatalog();

    Coord* px;
    Coord* py;
    int pn;
    int pt;
    gv->GetCurrent(px, py, pn, pt);
    const float scaleup = 1000;
    for (int i=0; i<pn; i++) {
      px[i] = (int)scaleup*px[i];
      py[i] = (int)scaleup*py[i];
    }

    if (rel != nil) {
      rel = new Transformer(rel);
      rel->Invert();
    }

    for (int i = 1; i < pn-1; i++) {
      px[i] = (px[i-1]+px[i]+px[i+1])/3;
      py[i] = (py[i-1]+py[i]+py[i+1])/3;
    }

    for (int i = 1; i < pn-1; i++) {
      px[i] = (px[i-1]+px[i]+px[i+1])/3;
      py[i] = (py[i-1]+py[i]+py[i+1])/3;
    }

    for (int i = 1; i < pn-1; i++) {
      px[i] = (px[i-1]+px[i]+px[i+1])/3;
      py[i] = (py[i-1]+py[i]+py[i+1])/3;
    }

    Coord x0, y0, x1, y1;
    x0 = px[0];
    y0 = py[0];
    for (int i = 1; i < pn-1; i++) {
      if (i!=1) printf(",");
      printf("(%d,%d)", px[i], py[i]);
      x0 = x1;
      y0 = y1;
      x1 = (px[i-1]+px[i]+px[i+1])/3;
      y1 = (py[i-1]+py[i]+py[i+1])/3;

      if (x0 != x1 || y0 != y1) {
	float fpx, fpy, ffx, ffy;
	
	rel->transform(x0/scaleup, y0/scaleup, fpx, fpy);
	rel->transform(x1/scaleup, y1/scaleup, ffx, ffy);

	// get line in ground coordinates
	// inverse project to obtain radian-based lat/long
	// set camera to those coordinates
	MapViewState* mapviewstate = ed->GetMapViewState();
	float dpx, dpy, dfx, dfy;
	if (Transformer* deftr = mapviewstate->deftrans()) {
	  deftr->inverse_transform(fpx, fpy, dpx, dpy);
	  deftr->inverse_transform(ffx, ffy, dfx, dfy);
	}

	PJ* proj = ed->GetProjection();
	if (!proj) // utmviewer
	    proj = mapviewstate->implicitproj();
	if (proj) {
	  float gpx, gpy, gfx, gfy;
	  UV data;
	  data.u = dpx;
	  data.v = dpy;
	  data = pj_inv(data, proj);
	  gpx = data.u;
	  gpy = data.v;

	  data.u = dfx;
	  data.v = dfy;
	  data = pj_inv(data, proj);
	  gfx = data.u;
	  gfy = data.v;

#if 0
	  printf("pos: x=%f, y=%f, degx=%f, degy=%f\n", gpx, gpy,
		 gpx * RAD_TO_DEG, gpy * RAD_TO_DEG);
	  printf("fpt: x=%f, y=%f, degx=%f, degy=%f\n", gfx, gfy,
		 gfx * RAD_TO_DEG, gfy * RAD_TO_DEG);
#endif

#if 0
	  if (ed->GetCameraComp()) {
	      OverlayView* view =
		((OverlayComp*)ed->GetCameraComp())->FindView(ed->GetViewer());
	      ArrowLineOvComp* comp = (ArrowLineOvComp*)ed->GetCameraComp();
	      ArrowLine* aline = comp->GetArrowLine();
	      aline->SetTransformer(rel);
	      aline->SetOriginal(x0, y0, x1, y1);
	      comp->Notify();
	      ed->GetViewer()->Update();
	  }
	  else {
	      Catalog* catalog = unidraw->GetCatalog();
	      ArrowLine* aline = new ArrowLine(x0, y0, x1, y1, false, true,
					       1.25, stdgraphic);
	      PSBrush* br = catalog->FindBrush(0xffff, 2);
	      aline->SetBrush(br);
	      PSColor* col = catalog->FindColor("Red");
	      aline->SetColors(col, col);
	      aline->SetTransformer(rel);
	      ArrowLineOvComp* comp = new ArrowLineOvComp(aline);
	      Command* pcmd = new PasteCmd(ed, new Clipboard(comp));
	      pcmd->Execute();
	      ed->GetSelection()->Clear();
	      ed->GetViewer()->Update();
	      ed->SetCameraComp(comp);
	  }
#endif
	  Command* ccmd = new SendCameraCmd(gpx, gpy, gfx, gfy);
	  ccmd->SetEditor(ed);
	  cmd->Append(ccmd);
	}
	else
	  printf("projection is nil\n");
      }
    }
    printf("\n");
    return cmd;
}
